<?php
namespace SpoonerWeb\DeployerInformation\ToolbarItem;

/*
 * This file is part of a TYPO3 extension.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Backend\Backend\ToolbarItems\SystemInformationToolbarItem;
use TYPO3\CMS\Core\Core\Environment;
use TYPO3\CMS\Core\Imaging\Icon;
use TYPO3\CMS\Core\Imaging\IconFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;

class LastDeployInformation
{

    /**
     * Deployment types with name as index
     * and path to the file as value
     *
     * @var array
     */
    protected $deploymentTypes = [
        'deployer' => '/../../.dep/releases',
        'custom' => '/LAST_DEPLOY'
    ];

    protected $projectPath = '';

    public function __construct()
    {
        if (class_exists(\TYPO3\CMS\Core\Core\Environment::class)) {
            $this->projectPath = \TYPO3\CMS\Core\Core\Environment::getProjectPath();
        } else {
            $this->projectPath = PATH_site;
        }
    }

    public function addLastDeployInformation(SystemInformationToolbarItem $systemInformationToolbarItem): void
    {
        $lastReleaseDateTime = $this->checkLastDeployInformation();
        if ($lastReleaseDateTime) {
            $systemInformationToolbarItem->addSystemInformation(
                'Last Deployment',
                strftime('%a, %x %H:%M', $lastReleaseDateTime->getTimestamp()),
                'deployer-information'
            );
        }
    }

    protected function checkLastDeployInformation(): ?\DateTime
    {
        $lastReleaseDateTime = null;
        foreach ($this->deploymentTypes as $releaseType => $releaseFilePath) {
            $absoluteReleaseFilePath = $this->projectPath . $releaseFilePath;
            if ($this->{'is' . ucfirst($releaseType) . 'Release'}($absoluteReleaseFilePath)) {
                $lastReleaseDateTime = $this->{'getLastReleaseDateOf' . ucfirst($releaseType) . 'Release'}($absoluteReleaseFilePath);
                break;
            }
        }

        return $lastReleaseDateTime;
    }

    private function isDeployerRelease(string $releaseFilePath): bool
    {
        return file_exists($releaseFilePath) && !empty(file($releaseFilePath));
    }

    private function getLastReleaseDateOfDeployerRelease(string $releaseFilePath): \DateTime
    {
        $releasesFileContent = file($releaseFilePath);

        $lastRelease = array_pop($releasesFileContent);
        $lastReleaseTime = explode(',', $lastRelease)[0];
        $lastReleaseDateTime = \DateTime::createFromFormat('YmdHis', $lastReleaseTime);

        return $lastReleaseDateTime;
    }

    private function isCustomRelease(string $releaseFilePath): bool
    {
        return file_exists($releaseFilePath);
    }

    private function getLastReleaseDateOfCustomRelease(string $releaseFilePath): \DateTime
    {
        $lastReleaseDateTime = new \DateTime();
        $lastReleaseDateTime->setTimestamp(filemtime($releaseFilePath));

        return $lastReleaseDateTime;
    }
}